' Lunar Lander For Nostalganauts
' Written for for the Colour Maximite 2
'   by vegipete
'   July 2020
'
' v1 - First try
' v2 - bug fix, allow high altitude, add pointer arrow, constants for parameters

mode 1,8  ' default but still...
cls

const PADWIDTH = 60
const GRAVITY = 0.1
const POWER = 0.5
const STARTFUEL = 100
const DELAY = 50

dim surface(800)

'***********************************
' Build the lander sprites.
' The sprite file only contains images of the lander straight up
' with and without rocket burn. For nice rotation, 18 more images
' are generated, every 5 degrees, from 5 to 90 degrees (clockwise.)
' The rest of the circle is done by mirroring the sprite sideways/
' vertically/both.
' The base image is loaded onto page 2.
' Then the 90` rotation is built across the page on a 40 pixel grid.
' Finally, the 38 total sprites are then yanked from the page.
page write 2
cls
load png "LanderPict.png"

' generate the rotations
for i = 1 to 18
  image rotate 0, 0,40,40,i*40, 0,i*5
  image rotate 0,40,40,40,i*40,40,i*5
next i

' populate the sprites
for i = 1 to 19
  sprite read i,(i-1)*40,0,40,40
  sprite read 20+i,(i-1)*40,40,40,40
next i

'***********************************
do
  page write 1
  cls

  text MM.HRES/2,10,"Lunar Lander for Nostalganauts","CT",4,1,rgb(brown)

  xlander = 400 ' lander coordinates
  ylander = 100
  vxlander = 0  ' lander velocity
  vylander = 0  ' positive UP!
  alander = 1   ' lander active sprite - so we hide the right one
  rlander = 1   ' lander rotation - 1 to 72 = 0 to 355 degrees
  fuel = 0
  xpad = 0      ' left edge of landing pad - set by CreateMoon

  DrawStars
  CreateMoon
  DrawMoon
  sprite show alander,xlander,ylander,1
  page copy 1,0,B
  done = 0

  do
    ' test for keypresses
    direction = 0
    burn = 0
    if keydown(0) then
      for i = 1 to keydown(0)
        if keydown(i) = 130 then direction =  1       ' left
        if keydown(i) = 131 then direction = -1       ' right
        if keydown(i) = 32 and fuel > 0 then burn = 1 ' engine on
      next i
    endif

    ' adjust rotation
    rlander = rlander + direction
    if rlander > 72 then rlander = rlander - 72
    if rlander <  1 then rlander = rlander + 72

    ' thrust
    if burn > 0 and fuel > 0 then
      fuel = fuel - 1
      angle = (rlander - 1) * PI / 36
      vxlander = vxlander - POWER * sin(angle)
      vylander = vylander + POWER * cos(angle)
    endif

    ' gravity
    vylander = vylander - GRAVITY

    ' display velocity
    print @(175,570) "Xvel: " ;: format(vxlander,8,3)
    print @(325,570) "Yvel: " ;: format(vylander,8,3)
    print @(475,570) "Fuel: " fuel "  ";

    ' new position
    xlander = xlander + vxlander
    if xlander < -30 then xlander = xlander + MM.HRES + 20
    if xlander > MM.HRES-10 then xlander = xlander - MM.HRES - 20

    ylander = ylander - vylander
'    if ylander < 0 then ylander = ylander + MM.VRES
'    if ylander > MM.VRES-1 then ylander = ylander - MM.VRES

    ShowLander(rlander,burn)
    page copy 1,0,B

    ' test for landing
    if xlander > -20 and xlander < 780 then ' no test when off screen
      if ylander > surface(xlander+20) - 32 then  ' touched the surface?
        ShowLander(rlander,0)   ' engines off
        if xlander > xpad - 5 and xlander < xpad + PADWIDTH - 35 then ' over landing pad?
          if rlander < 3 or rlander > 68 then  ' near vertical?
            if vylander > -2 then   ' low vertical velocity?
              text xlander+20,ylander-15,"Success!","CT",2,1,rgb(green)
              WinNoise
            else
              text xlander+20,ylander-40),"Crash!","CT",2,1,rgb(red)
              text xlander+20,ylander-15),"Too fast.","CT"
              CrashNoise
            endif
          else
            text xlander+20,ylander-40),"Crash!","CT",2,1,rgb(red)
            text xlander+20,ylander-15),"Too far off level.","CT"
            CrashNoise
          endif
        else
          text xlander+20,ylander-40),"Crash!","CT",2,1,rgb(red)
          text xlander+20,ylander-15),"Missed the pad.","CT"
          CrashNoise
        endif
        done = 1
        page copy 1,0
      endif
    endif

    if burn > 0 and fuel > 0 and not done then
      ThrustNoise
    else
      pause DELAY
    endif

  loop until done

  page write 0
  text MM.HRES/2,200,"Press [Enter] to go again.","CT"
  text MM.HRES/2,230,"Press [ESC] to go quit.","CT"

  do
    k = asc(inkey$)
    if k = 27 then end
    if k = 13 then exit do
  loop

  sprite hide alander

loop

'***********************************
sub ThrustNoise
  for i = 1 to DELAY
    play tone rnd*120,rnd*120
    pause 1
  next
  play tone 0,0
end sub

'***********************************
sub CrashNoise
  for i = 1 to 1000
    play tone rnd*250,rnd*250
    pause 1
  next
  play tone 0,0
end sub

'***********************************
sub WinNoise
  for i = 1 to 5
    play tone 500,500
    pause 100
    play tone 300,300
    pause 100
  next
  play tone 0,0
end sub

'***********************************
sub showlander(n,b)

  local x,y
  local offset = 0

  if b then offset = 20
  x = xlander+20 ' local copy to deal with sky high lander
  y = ylander

  box 0,0,MM.HRES,12,1,0,0   ' erase anything left on pointer row
  arrow$ = " "
  if y < -39 then
    y = -39
    arrow$ = chr$(146)
  endif

  if x < -5 then x = -5
  if x > MM.HRES+4 then x = MM.HRES+4
  text x,1, arrow$, "CT"

  sprite hide alander
  select case n
    case 1 to 19
      alander = n + offset 
      sprite show alander,xlander,y,1,0

    case 20 to 36
      alander = 38-n + offset
      sprite show alander,xlander,y+2,1,2

    case 37 to 55
      alander = n-36 + offset
      sprite show alander,xlander,y+2,1,3

    case 56 to 72
      alander = 74-n + offset
      sprite show alander,xlander,y,1,1
  end select
end sub

'***********************************
sub DrawMoon

  for i = 0 to MM.HRES-1
    line i,surface(i),i,MM.VRES-1,1,rgb(gray)
  next i

end sub

'***********************************
sub CreateMoon
  local height

  fuel = STARTFUEL

  xpad = int(rnd * 400) + 200 - PADWIDTH/2
  height = 500 - int(rnd * 100)  ' landing pad altitude

  for i = 0 to PADWIDTH
    surface(xpad + i) = height
  next i

  for i = xpad-1 to 0 step -1
    surface(i) = surface(i+1) + 4.8 - 10*rnd  ' slight bias up
    if surface(i) > MM.VRES-75 then surface(i) = MM.VRES-75
  next i

  for i = xpad + PADWIDTH to MM.HRES
    surface(i) = surface(i-1) + 4.8 - 10*rnd  ' slight bias up
    if surface(i) > MM.VRES-75 then surface(i) = MM.VRES-75
  next i
end sub

'***********************************
' Format a number and print it.
' Following is format(n,8,3)
' +123.123
'    0.000
' -123.123
sub format(number,width,dec)
  local n

  n = number

  if abs(n) < 0.001 then n = 0
  if n = 0 then
    print "   0.000";
    exit sub
  end if
  if n > 0 then work$ = "+"
  if n < 0 then work$ = "-" : n = abs(n)
  work$ = work$ + str$(n) + "000000"
  print left$(work$,width);

end sub
'***********************************
sub DrawStars

  for i = 1 to 200
    pixel 800*rnd,500*rnd,map(rnd*255)
  next i

end sub

'***********************************
'***********************************
'***********************************



















